# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4

"""
Implementation of 'fetch' command
"""

import sys
import os

from ..context import bugzilla_instance
from ..bugzilla import BugzillaError
from .. import ui
from .base import Base

class Command(Base):
    """Download attachment specified by ID"""

    def register(self, subparsers):
        """Register args parser for 'fetch' command"""
        parser = subparsers.add_parser('fetch')
        parser.set_defaults(func=self.run)
        parser.add_argument('attachment_id', type=int, help='attachment ID')
        parser.add_argument('-o', '--output', dest='output', \
            help='output filename, use - for stdout')

    def run(self, args):
        """Implementation for 'fetch' command"""
        bugzilla = bugzilla_instance()
        try:
            attachment = bugzilla.attachment(args.attachment_id)
        except BugzillaError as exc:
            ui.fatal('Bugzilla error: {}'.format(exc.message))

        if attachment is None:
            ui.fatal('attachment {} not found'.format(args.attachment_id))

        # Not None and not empty
        if args.output:
            file_name = args.output
        else:
            file_name = os.path.basename(attachment.file_name)
            orig_file_name = file_name
            # If file exists try to download to filename.N
            i = 1
            while os.path.exists(file_name):
                file_name = orig_file_name + '.' + str(i)
                i += 1

        desc_name = 'standard out' if file_name == '-' else file_name
        ui.output("Downloading attachment #{} to {}".format(attachment.object_id, desc_name))
        try:
            attachment = bugzilla.attachment(args.attachment_id, data=True)
        except BugzillaError as ex:
            ui.fatal('Bugzilla error: {}'.format(ex.message))

        try:
            if file_name == '-':
                out = sys.stdout.buffer
            else:
                out = open(file_name, 'wb+')
            out.write(attachment.data)
        except IOError as ex:
            ui.fatal('error saving file: {}'.format(str(ex)))



# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
"""
CLI execution context module. Handles data that can
be accessed by multiple components during CLI lifecycle
e.g.: settings, Bugzilla XMLRPC client.
"""

from .settings import Settings
from .bugzilla import Bugzilla

CONFIG_FILE = None

def set_default_config_file(path):
    global CONFIG_FILE
    CONFIG_FILE = path

def settings_instance():
    """Returns Settings singleton"""
    return Settings(CONFIG_FILE)

def bugzilla_instance():
    """Creates and returns Bugzilla XMLRPC client"""
    settings = settings_instance()
    bugzilla = Bugzilla(settings.url())
    bugzilla.set_api_key(settings.api_key())
    return bugzilla



# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4

"""
Implementatoin of the 'files' command
"""

from datetime import datetime, timezone

from ..bugzilla import BugzillaError
from ..context import bugzilla_instance
from .. import ui
from .base import Base

def format_date(date):
    """
    Format date based on current datetime.
    Include year if it's more than a year since the date
    """
    delta = datetime.utcnow().replace(tzinfo=timezone.utc) - date
    if delta.days < 364:
        return date.strftime('%b %d %H:%M')

    return date.strftime('%Y %b %d')


class Command(Base):
    """List files attached to specified PR"""

    def register(self, subparsers):
        parser = subparsers.add_parser('files')
        parser.set_defaults(func=self.run)
        parser.add_argument('bug_id', type=int, help='Bug ID')
        parser.add_argument('-a', '--all', action='store_true', \
            help='show all attachments (including obsolete)')
        parser.add_argument('-s', '--summary', action='store_true', \
            help='show summary instead of file name')

    def run(self, args):
        bugzilla = bugzilla_instance()
        try:
            attachments = bugzilla.attachments(args.bug_id, args.all)
        except BugzillaError as exc:
            ui.fatal('Bugzilla error: {}'.format(exc.message))

        rows = []
        for attachment in attachments:
            row = []
            row.append(str(attachment.object_id))
            row.append(attachment.creator)
            row.append(str(attachment.size))
            row.append(format_date(attachment.creation_time))
            if args.all:
                row.append('O' if attachment.is_obsolete else '')
            row.append(attachment.summary if args.summary else attachment.file_name)
            rows.append(row)

        # find width for every columen except last one
        # it's either file name or summary so should not be limited
        if rows:
            column_formats = []
            for i in range(len(rows[0]) - 1):
                width = max([len(str(row[i])) for row in rows])
                column_format = '{: >%d}' % width
                column_formats.append(column_format)
            row_format = '  '.join(column_formats)
            row_format += ' {}'
            for row in rows:
                ui.output(row_format.format(*row))



# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4

"""
'attach' command implementation'''
"""

from base64 import b64encode

import magic
from ..bugzilla import BugzillaError
from ..context import bugzilla_instance

from .. import ui
from .base import Base

class Command(Base):
    """Attach file to the existing PR"""

    def register(self, subparsers):
        """Register 'attach' parser"""
        parser = subparsers.add_parser('attach')
        parser.set_defaults(func=self.run)
        parser.add_argument('attachment', type=str, help='path to the attachment')
        parser.add_argument('pr', type=int, help='PR number')
        parser.add_argument('-s', '--summary', dest='summary', help='summary for the attachment')
        parser.add_argument('-c', '--comment', dest='comment', help='comment text')
        parser.add_argument('-t', '--content-type', dest='content_type', help='file content type')

    def run(self, args):
        """Run 'attach' command"""
        bugzilla = bugzilla_instance()
        content_type = args.content_type
        # Read data and encode it to base64
        try:
            with open(args.attachment, 'rb') as attach_file:
                data = attach_file.read()
        except IOError as ex:
            ui.fatal('error reading file: {}'.format(str(ex)))

        comment = args.comment
        if comment is None:
            comment = ui.edit_message()

        # Try and guess file content type
        if content_type is None:
            mime = magic.Magic(mime=True)
            content_type = mime.from_file(args.attachment)
        try:
            attachment = bugzilla.add_attachment(args.pr, args.attachment, data, \
                summary=args.summary, comment=comment, content_type=content_type)
        except BugzillaError as ex:
            ui.fatal('Bugzilla error: {}'.format(ex.message))
        ui.output('New attachment {} has been added to bug {}'.format(attachment, args.pr))
        ui.output('Attachment URL: {}'.format(bugzilla.attachment_url(attachment)))
        ui.output('Bug URL: {}'.format(bugzilla.bug_url(args.pr)))



# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
"""
Entry point for CLI utility
"""

from .commands import create_parser
from .context import set_default_config_file

def main():
    """Entry point function for scarab CLI"""
    parser = create_parser()
    args = parser.parse_args()
    if not args.config is None:
        set_default_config_file(args.config)

    if hasattr(args, 'func'):
        args.func(args)
    else:
        parser.print_help()

if __name__ == '__main__':
    main()



# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4

"""
Implementation of 'products' command
"""

from ..context import bugzilla_instance
from ..bugzilla import BugzillaError
from .. import ui
from .base import Base

class Command(Base):
    """List products information"""

    def register(self, subparsers):
        """Register parser for 'products' command"""
        parser = subparsers.add_parser('products')
        parser.set_defaults(func=self.run)

    def run(self, args):
        """Implement 'products' command"""

        bugzilla = bugzilla_instance()
        try:
            products = bugzilla.products()
        except BugzillaError as exc:
            ui.fatal('Bugzilla error: {}'.format(exc.message))

        for product in products:
            ui.output("Product '{}':".format(product.name))
            ui.output('  components:')
            for component in product.components:
                ui.output('    {}'.format(component.name))
            ui.output('  versions:')
            for version in product.versions:
                ui.output('    {}'.format(version.name))



# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
import pytest
import os

from scarab.settings import Settings

BARE_CONFIG = '''
[default]
url = server
api_key = key
'''

INVALID_CONFIG = '''
[default]
url = server
api_key = key

[template:a]
foo = bar
'''

FULL_CONFIG = '''
[default]
url = server
api_key = key

[template:a]
product = product a
component = component a
version = version a

[template:b]
version = version b
severity = severity b
'''

@pytest.yield_fixture
def bare_config_path(tmpdir):
    path = os.path.join(tmpdir, 'scarabrc')
    with open(path, 'w+') as f:
        f.write(BARE_CONFIG)
    yield path
    os.unlink(path)

@pytest.yield_fixture
def full_config_path(tmpdir):
    path = os.path.join(tmpdir, 'scarabrc')
    with open(path, 'w+') as f:
        f.write(FULL_CONFIG)
    yield path
    os.unlink(path)

@pytest.yield_fixture
def invalid_config_path(tmpdir):
    path = os.path.join(tmpdir, 'scarabrc')
    with open(path, 'w+') as f:
        f.write(INVALID_CONFIG)
    yield path
    os.unlink(path)

def test_defaults(monkeypatch):
    def mockexpand(path):
        return '/do/not/exist'
    monkeypatch.setattr(os.path, 'expanduser', mockexpand)
    settings = Settings()
    assert settings.url() is not None
    assert settings.api_key() is None

def test_valid_config(bare_config_path):
    settings = Settings(bare_config_path)
    assert settings.url() == 'server/'
    assert settings.api_key() == 'key'

def test_single_template(full_config_path):
    settings = Settings(full_config_path)
    template = settings.template('a')
    assert template['product'] == 'product a'
    assert template['component'] == 'component a'
    assert template['version'] == 'version a'

def test_combine_templates(full_config_path):
    settings = Settings(full_config_path)
    template = settings.combine_templates(['a', 'b'])
    assert template['product'] == 'product a'
    assert template['component'] == 'component a'
    assert template['version'] == 'version b'
    assert template['severity'] == 'severity b'

def test_invalid_template(invalid_config_path):
    with pytest.raises(Settings.InvalidTemplateKey) as ex:
        settings = Settings(invalid_config_path)
    assert ex.value.key == 'foo'
    assert ex.value.section == 'template:a'



# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
"""
Wrapper class for Bugzilla's XML-RPC API
"""

import xmlrpc.client
from datetime import datetime, timezone
import urllib

class BugzillaError(Exception):
    """
    Bugzilla errors like XML-RPC transport failures or method failures.
    """
    def __init__(self, message):
        super().__init__(message)
        self.message = message

def xmlrpc_method(method):
    """
    Catch-all wrapper for XML-RPC errors, should be applied
    to methods that use XML-RPC calls.
    """
    def wrapper(*args, **kwargs):
        """decorator's wrapper"""
        try:
            return method(*args, **kwargs)
        except xmlrpc.client.ProtocolError as ex:
            raise BugzillaError('{}: {} {}'.format(ex.url, \
                ex.errcode, ex.errmsg))
        except xmlrpc.client.Fault as ex:
            raise BugzillaError('{} ({})'.format(ex.faultString, ex.faultCode))

    return wrapper

class Attachment(object):
    """Bugzilla attachment representation"""
    def __init__(self, d):
        self.object_id = int(d['id'])
        self.bug_id = int(d['bug_id'])
        self.file_name = d['file_name']
        self.summary = d['summary']
        self.content_type = d['content_type']
        self.size = int(d['size'])
        time_tuple = d['creation_time'].timetuple()
        # Convert from UTC to local timezone
        self.creation_time = datetime(*time_tuple[0:6], tzinfo=timezone.utc).astimezone(tz=None)
        time_tuple = d['last_change_time'].timetuple()
        # Convert from UTC to local timezone
        self.last_change_time = datetime(*time_tuple[0:6], tzinfo=timezone.utc).astimezone(tz=None)
        self.creator = d['creator']
        self.is_obsolete = d['is_obsolete']
        if 'data' in d:
            self.data = d['data'].data
        else:
            self.data = b''

    def __repr__(self):
        return "Attachment(%d, '%s')" % (self.object_id, self.file_name)

class Product(object):
    """Bugzilla product representation"""

    class Component(object):
        """Bugzilla product's component representation"""
        def __init__(self, d):
            self.object_id = int(d['id'])
            self.name = d['name']

        def __repr__(self):
            return "Component(%d, '%s')" % (self.object_id, self.name)

    class Version(object):
        """Bugzilla product's version representation"""
        def __init__(self, d):
            self.object_id = int(d['id'])
            self.name = d['name']

        def __repr__(self):
            return "Component(%d, '%s')" % (self.object_id, self.name)

    def __init__(self, d):
        self.object_id = int(d['id'])
        self.name = d['name']
        self.description = d['description']
        self.components = []
        self.versions = []
        for component_d in d['components']:
            self.components.append(self.Component(component_d))
        for version_d in d['versions']:
            self.versions.append(self.Version(version_d))

    def __repr__(self):
        return "Product(%d, '%s')" % (self.object_id, self.name)

class Bugzilla(object):
    """Wrapper for Bugzilla's XML-RPC API"""
    __api_key = None

    def __init__(self, url):
        self.__base_url = url
        if not self.__base_url.endswith('/'):
            self.__base_url += '/'
        xmlrpc_url = self.__url('xmlrpc.cgi')
        self.__proxy = xmlrpc.client.ServerProxy(xmlrpc_url)

    def set_api_key(self, api_key):
        """Set API key for current session"""
        self.__api_key = api_key

    def bug_url(self, bug):
        url = 'show_bug.cgi?id={}'.format(bug)
        return self.__url(url)

    def attachment_url(self, attachment):
        url = 'attachment.cgi?id={}&action=edit'.format(attachment)
        return self.__url(url)

    def __url(self, relative_url):
        """Returns absolute URL for relative_url using bugzilla URL as a base"""
        return urllib.parse.urljoin(self.__base_url, relative_url)

    def __common_args(self):
        """Initialize part of parameters common for all XML-RPC methods"""
        if self.__api_key is None:
            return {}

        return {'Bugzilla_api_key': self.__api_key}

    @xmlrpc_method
    def attachments(self, bug_id, include_obsolete=False):
        """
        Get list of attachment for specified bug_id
        Args:
            bug_id (int): bug ID
            include_obsolete (boolean, optional):
                if provided and set to True, include attachments
                marked as obsolete into result
        Returns:
            list of Attachment objects representing files attached
            to the specified bug. Returns empty list of there are none.
        """
        args = self.__common_args()
        args['ids'] = [bug_id]
        # Do not requets attachment data
        args['exclude_fields'] = ['data']
        reply = self.__proxy.Bug.attachments(args)
        result = []
        for attachment in reply['bugs'][str(bug_id)]:
            result.append(Attachment(attachment))
        if not include_obsolete:
            result = [a for a in result if not a.is_obsolete]
        result.sort(key=lambda a: a.object_id)

        return result

    @xmlrpc_method
    def attachment(self, attachment_id, data=False):
        """
        Get Attachment object for specified attachment_id. Returns
        None if attachment_id does not exist
        """
        args = self.__common_args()
        args['attachment_ids'] = [attachment_id]
        if not data:
            args['exclude_fields'] = ['data']
        reply = self.__proxy.Bug.attachments(args)
        if not str(attachment_id) in reply['attachments']:
            return None
        return Attachment(reply['attachments'][str(attachment_id)])

    @xmlrpc_method
    def add_attachment(self, bug_id, file_name, data, \
      summary=None, comment=None, content_type='application/octect-stream'):
        """
        Add attachment to specified bug.
        Args:
            bug_id (int): bug ID
            file_name (str): Remote filename (not a local path to the file).
                To be stored as a part of attachment metadata. Should not
                contain path elements.
            data (str): base64-encoded content of the file
            summary (str, optional): one-line description of the attachment
            comment (str, optional): comment to be posted along with the attachment
            content_type (str, optional): content-type of the attachment.
                If not provided application/octet-stream is assumed
        """
        args = self.__common_args()
        args['ids'] = [bug_id]
        args['file_name'] = file_name
        args['data'] = data
        args['summary'] = summary if summary is not None else file_name
        args['content_type'] = content_type
        if comment is not None:
            args['comment'] = comment
        reply = self.__proxy.Bug.add_attachment(args)
        ids = reply.get('ids', [])
        if ids:
            return ids[0]
        return None

    @xmlrpc_method
    def products(self):
        """
        Returns list of Product objects representing
        products to which user can submit bugs
        """
        args = self.__common_args()
        ids = self.__proxy.Product.get_accessible_products(args)
        reply = self.__proxy.Product.get(ids)
        result = []
        for obj in reply['products']:
            product = Product(obj)
            result.append(product)
        return result

    @xmlrpc_method
    def submit(self, product, component, version, summary, \
            description=None, cc_list=None, platform=None,
            severity=None):
        """
        Submit new bug
        Args:
            product (string): product name
            component (string): component name
            version (string): version value
            summary (string): one-line summary of the bug
            description (string, optional): description of the bug
            cc_list (list, optional): list of users' emails to add to Cc list of the bug
            severity (string, optional): bug severity
            platform (string, optional): platform where bug was found
        """
        args = self.__common_args()
        args['product'] = product
        args['component'] = component
        args['summary'] = summary
        args['version'] = version
        if description:
            args['description'] = description
        if cc_list:
            args['cc'] = cc_list
        if platform:
            args['platform'] = platform
        if severity:
            args['severity'] = severity
        reply = self.__proxy.Bug.create(args)
        return reply['id']



# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4

"""
Implementation of 'submit' command
"""

from ..context import bugzilla_instance, settings_instance
from ..settings import Settings
from ..bugzilla import BugzillaError
from .. import ui
from .base import Base

class Command(Base):
    """Submit new PR"""

    def register(self, subparsers):
        """Register parser for 'submit' command"""
        parser = subparsers.add_parser('submit')
        parser.set_defaults(func=self.run)
        parser.add_argument('-t', '--template', dest='templates', \
            action='append', help='name of the pre-configured bug template')
        parser.add_argument('-p', '--product', dest='product', \
            help='name of the product')
        parser.add_argument('-m', '--component', dest='component', \
            help='name of the component')
        parser.add_argument('-v', '--version', dest='version', \
            help='version value')
        parser.add_argument('-c', '--comment', dest='comment', \
            help='comment describing the bug')
        parser.add_argument('-s', '--summary', dest='summary', \
            required=True, help='summary for the bug')
        parser.add_argument('-C', '--cc', dest='cc', \
            action='append', help='users to add to CC list (can be specified multiple times)')
        parser.add_argument('-F', '--platform', dest='platform', help='platform')
        parser.add_argument('-S', '--severity', dest='severity', help='severity')

    def run(self, args):
        """Implement 'submit' command"""

        template = None
        try:
            if args.templates:
                template = settings_instance().combine_templates(args.templates)
        except Settings.TemplateNotFound as ex:
            ui.fatal("Template '{}' is not defined".format(ex.template_name))

        product = None
        component = None
        version = None
        severity = None
        platform = None

        if template:
            product = template.get('product', None)
            component = template.get('component', None)
            version = template.get('version', None)
            severity = template.get('severity', None)
            platform = template.get('platform', None)

        # Values specified from command line override
        # values from template
        if args.product:
            product = args.product
        if args.component:
            component = args.component
        if args.version:
            version = args.version
        if args.severity:
            severity = args.severity
        if args.platform:
            platform = args.platform

        # product, component and version
        if product is None:
            ui.fatal('product value was not specified')
        if component is None:
            ui.fatal('component value was not specified')
        if version is None:
            ui.fatal('version value was not specified')

        summary = args.summary
        cc_list = args.cc

        comment = args.comment
        if comment is None:
            comment = ui.edit_message()

        bugzilla = bugzilla_instance()
        try:
            bug = bugzilla.submit(product, component, version, summary, \
                description=comment, cc_list=cc_list, severity=severity, \
                platform=platform)
        except BugzillaError as exc:
            ui.fatal('Bugzilla error: {}'.format(exc.message))

        ui.output('New bug {} has been submitted'.format(bug))
        ui.output('Bug URL: {}'.format(bugzilla.bug_url(bug)))



# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
"""
Module handles all the interactoins with user
"""
import sys, tempfile, os
from subprocess import call

MESSAGE = '''
# SCARAB: Please enter the comment. Lines starting with '# SCARAB:'
# SCARAB: will be ignored
'''

def fatal(line):
    """
    Handle fatal error: log message and abort execution
    """
    sys.stderr.write('{}\n'.format(line))
    sys.exit(1)

def log(line):
    """Log debug/error message"""
    sys.stderr.write('{}\n'.format(line))

def output(line):
    """Output normal message"""
    sys.stdout.write('{}\n'.format(line))

def edit_message(initial_message = None):
    """
    Invoke editor to let user provide multi-line input. All lines starting
    with '# SCARAB:' are considered internal comments and ignored. Leading
    and trailing spaces are removed from final comment
    """
    editor = os.environ.get('SCARAB_EDITOR', None)
    if editor is None:
        editor = os.environ.get('EDITOR', 'vi')

    tf = tempfile.NamedTemporaryFile(suffix=".tmp", mode="w+", delete=False)
    if not initial_message is None:
        tf.write(initial_message)
    tf.write(MESSAGE)
    tf.flush()
    name = tf.name
    tf.close()

    call([editor, name])

    with open(name, 'r') as tf:
        message = tf.read()

    os.unlink(name)

    lines = message.split('\n')
    lines = [l for l in lines if not l.startswith('# SCARAB:')]
    message = '\n'.join(lines).strip()

    return message



# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4

"""
Base class for CLI command. All CLI commands are modules in
scarab.commands.* packag and should implement scarab.commands.Base
interface: register and run methods.
"""

class Base(object):
    """A base command."""

    def __init__(self):
        pass

    def register(self, subparsers):
        """
        Creates and registers command parser using add_parser method.
        Should be re-implemented in child class
        """
        raise NotImplementedError('You must implement the register() method yourself!')


    def run(self, args):
        """
        Actual implementation of the command functionality. args is argparse
        params object.
        Should be re-implemented in child class
        """
        raise NotImplementedError('You must implement the run() method yourself!')



# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4

"""
Top-level CLI command package
"""
import argparse
import traceback

from importlib import import_module
from pkgutil import walk_packages

from .. import ui

def create_parser():
    """
    Autoload all available modules in the package and check if they're
    command implementation.
    Returns:
        argparse.ArgumentParser object with all available subcommands
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('-c', '--config', dest='config', help='config file')
    subparsers = parser.add_subparsers(title="commands", dest="command")
    for entry in walk_packages(__path__, __name__ + '.'):
        try:
            module = import_module(entry[1])
        except Exception as exc:
            ui.log("Error importing module '{}': {}".format(entry[1], exc))
            ui.log(traceback.format_exc())
            continue

        if hasattr(module, 'Command'):
            command = module.Command()
            command.register(subparsers)

    return parser



# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
"""
Representation of scarab's run-time settings
"""

import configparser
import os

from . import ui

class Settings(object):
    class TemplateNotFound(Exception):
        def __init__(self, template_name):
            self.template_name = template_name

    class InvalidTemplateKey(Exception):
        def __init__(self, key, section):
            self.key = key
            self.section = section

    """Singleton class that provides access to run-time settings"""
    __instances = {}
    VALID_TEMPLATE_KEYS = [
        'product',
        'component',
        'version',
        'severity',
        'platform',
    ]
    def __new__(cls, config_file=None):
        if config_file is None:
            home = os.path.expanduser('~')
            path = os.path.join(home, '.scarabrc')
            if os.path.exists(path):
                config_file = path

        if config_file in Settings.__instances:
            return Settings.__instances[config_file]

        instance = object.__new__(cls)
        instance.__config = configparser.ConfigParser()
        if not config_file is None:
            try:
                instance.load_file(config_file)
            except IOError as ex:
                ui.fatal("Error reading config file:\n{}".format(ex))
            except configparser.Error as ex:
                ui.fatal("Error reading config file:\n{}".format(ex))

        Settings.__instances[config_file] = instance
        return instance

    def load_file(self, path):
        """Load ini file specified by path"""
        self.__config.read_file(open(path))
        self.__templates = {}
        # Parse template
        for section in self.__config:
            if not section.startswith('template:'):
                continue
            template_name = section[9:]
            template = {}
            for key in self.__config[section]:
                if not key in self.VALID_TEMPLATE_KEYS:
                    raise self.InvalidTemplateKey(key, section)
                template[key] = self.__config[section][key]
                self.__templates[template_name] = template

    def url(self):
        """
        Returns bugzilla base URL configured in [default]
        section of the config file (parameter 'url')
        """
        base = self.__config.get('default', 'url', \
            fallback='https://bugs.freebsd.org/bugzilla/')
        if not base.endswith('/'):
            base += '/'
        return base

    def api_key(self):
        """
        Returns API key configured in [global] sectoin of
        the config file (parameter 'api_key')
        """
        return self.__config.get('default', 'api_key', fallback=None)

    def template(self, name):
        """
        Returns dict with values from template 'name' or raises KeyError
        if it doesn't exist
        """
        if name in self.__templates:
            return self.__templates[name]

        raise self.TemplateNotFound(name)

    def combine_templates(self, names):
        """
        Returns dict with values from template 'name' or None if template
        with such name does not exist
        """
        result = {}
        for name in names:
            template = self.template(name)
            result.update(template)

        return result



# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4

"""
Implementation of fetchall command
"""

import os

from .base import Base
from ..context import bugzilla_instance
from ..bugzilla import BugzillaError
from .. import ui

class Command(Base):
    """Download all non-obsolete attachement for specified bug ID"""

    def register(self, subparsers):
        """Register parser for 'fetchall' command"""
        parser = subparsers.add_parser('fetchall')
        parser.set_defaults(func=self.run)
        parser.add_argument('bug_id', type=int, help='bug ID')

    def run(self, args):
        """Implementation of the 'fetchall' command"""
        bugzilla = bugzilla_instance()
        try:
            attachments = bugzilla.attachments(args.bug_id)
        except BugzillaError as ex:
            ui.fatal('Bugzilla error: {}'.format(ex.message))

        for attachment in attachments:
            file_name = os.path.basename(attachment.file_name)
            orig_file_name = file_name
            # If file exists try to download to filename.N
            i = 1
            while os.path.exists(file_name):
                file_name = orig_file_name + '.' + str(i)
                i += 1

            ui.output("Downloading attachment #{} to {}".format(attachment.object_id, file_name))
            try:
                attachment = bugzilla.attachment(attachment.object_id, data=True)
            except BugzillaError as ex:
                ui.fatal('Bugzilla error: {}'.format(ex.message))

            try:
                out = open(file_name, 'wb+')
                out.write(attachment.data)
            except IOError as ex:
                ui.fatal('error saving file: {}'.format(str(ex)))
